'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _private_ = exports._private_ = new WeakMap();

/**
 * Utility methods of Web Component
 */

var Component = function () {
    function Component() {
        _classCallCheck(this, Component);
    }

    _createClass(Component, [{
        key: 'boot',


        /**
         * @protected
         *
         * @param {DocumentFragment} template - `HTMLTemplateElement.prototype.content`
         *
         * @return {HTMLElement} This custom element
         */
        value: function boot(template) {

            this.attachShadow({
                mode: 'open',
                delegatesFocus: true
            }).appendChild(template.cloneNode(true));

            return this;
        }

        /**
         * Define a set of Getter or Setter for DOM properties,
         * and store their values into private object.
         *
         * @param {Object} map - `1` for Getter, `2` for Setter & sum for both
         *                       in each key's value
         * @return {string[]} Keys of `map`
         *
         * @example
         *
         *    WebCell.component(class MyInput extends HTMLElement {
         *
         *        constructor() {  super();  }
         *
         *        static get observedAttributes() {
         *
         *            return this.setAccessor({
         *                value:  1,
         *                step:   3
         *            });
         *        }
         *    });
         */

    }, {
        key: 'attributeChangedCallback',
        value: function attributeChangedCallback(name, oldValue, newValue) {

            switch (newValue) {
                case '':
                    this[name] = true;break;
                case null:
                    this[name] = false;break;
                default:
                    try {
                        this[name] = JSON.parse(newValue);
                    } catch (error) {

                        this[name] = newValue;
                    }
            }
        }

        /**
         * @param {Element} element
         *
         * @return {number} The index of `element` in its siblings
         */

    }, {
        key: '$',


        /**
         * @param {string} selector
         *
         * @return {Element[]} Element set which matches `selector` in this Shadow DOM
         */
        value: function $(selector) {

            return [].concat(_toConsumableArray(this.shadowRoot.querySelectorAll(selector)));
        }
    }], [{
        key: 'setAccessor',
        value: function setAccessor(map) {
            var _this = this;

            var _loop = function _loop(key) {

                var config = { enumerable: true };

                if (map[key] & 1) config.get = function () {

                    return _private_.get(this)[key];
                };

                if (map[key] & 2) config.set = function (value) {

                    _private_.get(this)[key] = value;
                };

                Object.defineProperty(_this.prototype, key, config);
            };

            for (var key in map) {
                _loop(key);
            }

            return Object.keys(map);
        }
    }, {
        key: 'indexOf',
        value: function indexOf(element) {

            var index = 0;

            while (element = element.previousElementSibling) {
                index++;
            }return index;
        }

        /**
         * @param {Event} event
         *
         * @return {Element} The target of `event` object (**Shadow DOM** is in account)
         */

    }, {
        key: 'targetOf',
        value: function targetOf(event) {

            var target = event.composedPath ? event.composedPath() : event.path;

            return (target || '')[0] || event.target;
        }

        /**
         * @protected
         *
         * @return   {Object}
         * @property {HTMLTemplateElement}                      template
         * @property {Array<HTMLStyleElement, HTMLLinkElement>} style
         * @property {HTMLScriptElement}                        script
         */

    }, {
        key: 'findTemplate',
        value: function findTemplate() {

            var script = document.currentScript,
                template,
                style = [];

            var element = script,
                stop;

            while (!stop && (element = element.previousElementSibling)) {
                switch (element.tagName.toLowerCase()) {
                    case 'template':
                        template = element;break;
                    case 'style':
                        style.unshift(element);break;
                    case 'link':
                        if (element.rel === 'stylesheet') {

                            element.setAttribute('href', element.href);

                            style.unshift(element);
                        }
                        break;
                    case 'script':
                        stop = true;
                }
            }return { template: template, style: style, script: script };
        }
    }, {
        key: 'tagName',

        /**
         * @type {string} - `tagName` of a Custom Element
         */
        get: function get() {

            return this.name.replace(/[A-Z]/g, function (char) {
                return '-' + char.toLowerCase();
            }).slice(1);
        }
    }]);

    return Component;
}();

exports.default = Component;