'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var view_DOM = new WeakMap(),
    DOM_view = new WeakMap(),
    view_data = new WeakMap();

/**
 * Abstract View
 */

var View = function (_Array) {
    _inherits(View, _Array);

    /**
     * @param {string|Element|DocumentFragment} template
     * @param {string}                          nameKey  - Key (littleCamelCase) of HTML `data-*`
     *                                                     to get name of bound data
     * @param {Object}                          scope    - Empty Model for this view
     */
    function View(template, nameKey, scope) {
        var _this;

        _classCallCheck(this, View);

        if ((_this = _possibleConstructorReturn(this, (View.__proto__ || Object.getPrototypeOf(View)).call(this)), _this).constructor === View) throw TypeError('"View" is an abstract class');

        switch (template.nodeType) {
            case 1:
                _this.name = template.dataset[nameKey];break;
            case 11:
                if (!template.parentNode) template = Array.from(template.childNodes);
                break;
            default:
                template = View.parseDOM(template);
        }

        var _this_ = _this.bindWith(template);

        if (_this_ !== _this) {
            var _ret;

            _this_.booted = true;

            return _ret = _this_, _possibleConstructorReturn(_this, _ret);
        }

        view_data.set(_this, scope);
        return _possibleConstructorReturn(_this);
    }

    /**
     * @param {Node} node - Node from other `Document`
     *
     * @return {Node} Cloned & imported Node (with its child nodes)
     */


    _createClass(View, [{
        key: 'bindWith',


        /**
         * @protected
         *
         * @param {Element|Element[]|DocumentFragment} template
         *
         * @return {View} This view or the view bound before
         */
        value: function bindWith(template) {
            var _this2 = this;

            var _this_;

            if (template instanceof Array) template = template.filter(function (node) {

                switch (node.nodeType) {
                    case 1:
                        if (!(_this_ = DOM_view.get(node))) DOM_view.set(node, _this2);
                        break;
                    case 3:
                        if (!node.nodeValue.trim()) return;
                }

                return true;
            });else if (!(_this_ = DOM_view.get(template))) DOM_view.set(template, this);

            view_DOM.set(this, template);

            return _this_ || this;
        }

        /**
         * @param {Element|DocumentFragment} node
         *
         * @return {View} View instance bound with `node`
         */

    }, {
        key: 'toString',


        /**
         * @return {string} Full markup code of this view
         */
        value: function toString() {

            var content = this.content;

            return content.nodeType === 1 ? content.outerHTML : Array.from(content.childNodes || content, function (node) {

                switch (node.nodeType) {
                    case 1:
                        return node.outerHTML;
                    case 3:
                        return node.nodeValue;
                }
            }).join('');
        }

        /**
         * @protected
         *
         * @type {Object}
         */

    }, {
        key: 'valueOf',


        /**
         * Get original data of this view
         *
         * @return {Object}
         */
        value: function valueOf() {

            throw TypeError('View.prototype.valueOf() must be overwriten');
        }

        /**
         * Render this view
         *
         * @return {View}
         */

    }, {
        key: 'render',
        value: function render() {

            throw TypeError('View.prototype.render() must be overwriten');
        }

        /**
         * Reset this view to empty data
         *
         * @return {View}
         */

    }, {
        key: 'clear',
        value: function clear() {

            throw TypeError('View.prototype.clear() must be overwriten');
        }
    }, {
        key: 'content',


        /**
         * @return {Element|Element[]|DocumentFragment}
         */
        get: function get() {
            return view_DOM.get(this);
        }
    }, {
        key: 'data',
        get: function get() {
            return view_data.get(this);
        }
    }], [{
        key: 'import',
        value: function _import(node) {
            return document.importNode(node, true);
        }

        /**
         * @param {string} markup - Code of an markup fragment
         *
         * @return {Element[]}
         */

    }, {
        key: 'parseDOM',
        value: function parseDOM(markup) {

            markup = new window.DOMParser().parseFromString(markup, 'text/html');

            return Array.from(markup.head.childNodes, View.import).concat(Array.from(markup.body.childNodes, View.import));
        }
    }, {
        key: 'instanceOf',
        value: function instanceOf(node) {
            return DOM_view.get(node);
        }
    }]);

    return View;
}(Array);

exports.default = View;