'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.component = component;
exports.set = set;
exports.get = get;

var _Component = require('./Component');

var _Component2 = _interopRequireDefault(_Component);

var _utility = require('./utility');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Register a component
 *
 * @param {function} subClass
 *
 * @return {function} A proxy class of `subClass`
 */
function component(subClass) {
    var _template;

    var _Component$findTempla = _Component2.default.findTemplate(),
        template = _Component$findTempla.template,
        style = _Component$findTempla.style;

    template = template.content;

    if (style[0]) (_template = template).prepend.apply(_template, _toConsumableArray(style));

    subClass = new Proxy((0, _utility.extend)(subClass, _Component2.default), {
        construct: function construct(target, argument, newTarget) {

            var _this_ = Reflect.construct(target, argument, newTarget);

            _Component._private_.set(_this_, {});

            return _this_.boot(template);
        }
    });

    customElements.define(subClass.tagName, subClass);

    return subClass;
}

/**
 * Set private data of an HTMLElement
 *
 * @param {Element} element
 * @param {string}  key
 * @param {*}       value
 *
 * @return {*} The `value` parameter
 */
function set(element, key, value) {

    return _Component._private_.get(element)[key] = value;
}

/**
 * Get private data of an HTMLElement
 *
 * @param {Element} element
 * @param {string}  key
 *
 * @return {*}
 */
function get(element, key) {

    return _Component._private_.get(element)[key];
}